@@ -19,21 +19,6 @@ package org.apache.maven.plugins.shade.mojo;
  * under the License.
  */
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.Writer;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.LinkedHashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.factory.ArtifactFactory;
 import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
@@ -48,8 +33,11 @@ import org.apache.maven.model.Model;
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugins.shade.Shader;
+import org.apache.maven.plugins.shade.filter.Filter;
+import org.apache.maven.plugins.shade.filter.MinijarFilter;
 import org.apache.maven.plugins.shade.filter.SimpleFilter;
 import org.apache.maven.plugins.shade.pom.PomWriter;
+import org.apache.maven.plugins.shade.relocation.Relocator;
 import org.apache.maven.plugins.shade.relocation.SimpleRelocator;
 import org.apache.maven.plugins.shade.resource.ResourceTransformer;
 import org.apache.maven.project.MavenProject;
@@ -59,10 +47,31 @@ import org.apache.maven.project.ProjectBuildingException;
 import org.apache.maven.shared.dependency.tree.DependencyNode;
 import org.apache.maven.shared.dependency.tree.DependencyTreeBuilder;
 import org.apache.maven.shared.dependency.tree.DependencyTreeBuilderException;
-import org.codehaus.plexus.util.FileUtils;
+import org.codehaus.plexus.PlexusConstants;
+import org.codehaus.plexus.PlexusContainer;
+import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
+import org.codehaus.plexus.context.Context;
+import org.codehaus.plexus.context.ContextException;
+import org.codehaus.plexus.personality.plexus.lifecycle.phase.Contextualizable;
 import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.WriterFactory;
 
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.Writer;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
 /**
  * Mojo that performs shading delegating to the Shader component.
  *
@@ -73,12 +82,14 @@ import org.codehaus.plexus.util.WriterFactory;
  * @goal shade
  * @phase package
  * @requiresDependencyResolution runtime
+ * @threadSafe
  */
 public class ShadeMojo
     extends AbstractMojo
+    implements Contextualizable
 {
     /**
-     * @parameter expression="${project}"
+     * @parameter default-value="${project}"
      * @readonly
      * @required
      */
@@ -92,7 +103,7 @@ public class ShadeMojo
     private MavenProjectHelper projectHelper;
 
     /**
-     * @component
+     * @component role="org.apache.maven.plugins.shade.Shader" roleHint="default"
      * @required
      * @readonly
      */
@@ -110,7 +121,7 @@ public class ShadeMojo
     /**
      * ProjectBuilder, needed to create projects from the artifacts.
      *
-     * @component role="org.apache.maven.project.MavenProjectBuilder"
+     * @component
      * @required
      * @readonly
      */
@@ -137,7 +148,7 @@ public class ShadeMojo
     /**
      * Remote repositories which will be searched for source attachments.
      *
-     * @parameter expression="${project.remoteArtifactRepositories}"
+     * @parameter default-value="${project.remoteArtifactRepositories}"
      * @required
      * @readonly
      */
@@ -146,7 +157,7 @@ public class ShadeMojo
     /**
      * Local maven repository.
      *
-     * @parameter expression="${localRepository}"
+     * @parameter default-value="${localRepository}"
      * @required
      * @readonly
      */
@@ -155,7 +166,7 @@ public class ShadeMojo
     /**
      * Artifact factory, needed to download source jars for inclusion in classpath.
      *
-     * @component role="org.apache.maven.artifact.factory.ArtifactFactory"
+     * @component
      * @required
      * @readonly
      */
@@ -164,40 +175,84 @@ public class ShadeMojo
     /**
      * Artifact resolver, needed to download source jars for inclusion in classpath.
      *
-     * @component role="org.apache.maven.artifact.resolver.ArtifactResolver"
+     * @component
      * @required
      * @readonly
      */
     protected ArtifactResolver artifactResolver;
 
     /**
-     * Artifacts to include/exclude from the final artifact.
+     * Artifacts to include/exclude from the final artifact. Artifacts are denoted by composite identifiers of the
+     * general form <code>groupId:artifactId:type:classifier</code>. Since version 1.3, the wildcard characters '*' and
+     * '?' can be used within the sub parts of those composite identifiers to do pattern matching. For convenience, the
+     * syntax <code>groupId</code> is equivalent to <code>groupId:*:*:*</code>, <code>groupId:artifactId</code> is
+     * equivalent to <code>groupId:artifactId:*:*</code> and <code>groupId:artifactId:classifier</code> is equivalent to
+     * <code>groupId:artifactId:*:classifier</code>. For example:
+     * <pre>
+     * &lt;artifactSet&gt;
+     *   &lt;includes&gt;
+     *     &lt;include&gt;org.apache.maven:*&lt;/include&gt;
+     *   &lt;/includes&gt;
+     *   &lt;excludes&gt;
+     *     &lt;exclude&gt;*:maven-core&lt;/exclude&gt;
+     *   &lt;/excludes&gt;
+     * &lt;/artifactSet&gt;
+     * </pre>
      *
      * @parameter
      */
     private ArtifactSet artifactSet;
 
     /**
-     * Packages to be relocated.
+     * Packages to be relocated. For example:
+     * <pre>
+     * &lt;relocations&gt;
+     *   &lt;relocation&gt;
+     *     &lt;pattern&gt;org.apache&lt;/pattern&gt;
+     *     &lt;shadedPattern&gt;hidden.org.apache&lt;/shadedPattern&gt;
+     *     &lt;includes&gt;
+     *       &lt;include&gt;org.apache.maven.*&lt;/include&gt;
+     *     &lt;/includes&gt;
+     *     &lt;excludes&gt;
+     *       &lt;exclude&gt;org.apache.maven.Public*&lt;/exclude&gt;
+     *     &lt;/excludes&gt;
+     *   &lt;/relocation&gt;
+     * &lt;/relocations&gt;
+     * </pre>
+     * <em>Note:</em> Support for includes exists only since version 1.4.
      *
      * @parameter
      */
     private PackageRelocation[] relocations;
 
     /**
-     * Resource transformers to be used.
+     * Resource transformers to be used. Please see the "Examples" section for more information on available
+     * transformers and their configuration.
      *
      * @parameter
      */
     private ResourceTransformer[] transformers;
 
     /**
-     * Archive Filters to be used.  Allows you to specify an artifact in the form of
-     * groupId:artifactId and a set of include/exclude file patterns for filtering which
-     * contents of the archive are added to the shaded jar.  From a logical perspective,
-     * includes are processed before excludes, thus it's possible to use an include to
-     * collect a set of files from the archive then use excludes to further reduce the set.
-     * By default, all files are included and no files are excluded.
+     * Archive Filters to be used. Allows you to specify an artifact in the form of a composite identifier as used by
+     * {@link #artifactSet} and a set of include/exclude file patterns for filtering which contents of the archive are
+     * added to the shaded jar. From a logical perspective, includes are processed before excludes, thus it's possible
+     * to use an include to collect a set of files from the archive then use excludes to further reduce the set. By
+     * default, all files are included and no files are excluded. If multiple filters apply to an artifact, the
+     * intersection of the matched files will be included in the final JAR. For example:
+     * <pre>
+     * &lt;filters&gt;
+     *   &lt;filter&gt;
+     *     &lt;artifact&gt;junit:junit&lt;/artifact&gt;
+     *     &lt;includes&gt;
+     *       &lt;include&gt;org/junit/**&lt;/include&gt;
+     *     &lt;/includes&gt;
+     *     &lt;excludes&gt;
+     *       &lt;exclude&gt;org/junit/experimental/**&lt;/exclude&gt;
+     *     &lt;/excludes&gt;
+     *   &lt;/filter&gt;
+     * &lt;/filters&gt;
+     * </pre>
      *
      * @parameter
      */
@@ -211,7 +266,11 @@ public class ShadeMojo
     private File outputDirectory;
 
     /**
-     * The name of the shaded artifactId
+     * The name of the shaded artifactId.
+     * <p/>
+     * If you like to change the name of the native artifact, you may use the &lt;build>&lt;finalName> setting.
+     * If this is set to something different than &lt;build>&lt;finalName>, no file replacement
+     * will be performed, even if shadedArtifactAttached is being used.
      *
      * @parameter expression="${finalName}"
      */
@@ -248,11 +307,21 @@ public class ShadeMojo
      * Flag whether to generate a simplified POM for the shaded artifact. If set to <code>true</code>, dependencies that
      * have been included into the uber JAR will be removed from the <code>&lt;dependencies&gt;</code> section of the
      * generated POM. The reduced POM will be named <code>dependency-reduced-pom.xml</code> and is stored into the same
-     * directory as the shaded artifact.
+     * directory as the shaded artifact. Unless you also specify dependencyReducedPomLocation, the plugin will
+     * create a temporary file named <code>dependency-reduced-pom.xml</code> in the project basedir. 
      *
      * @parameter expression="${createDependencyReducedPom}" default-value="true"
      */
     private boolean createDependencyReducedPom;
+    
+    
+    /**
+     *  @parameter expression="${dependencyReducedPomLocation}" defaultValue="${basedir}/dependency-reduced-pom.xml"
+     *  Note: setting a value for this parameter with a directory other than ${basedir} will change the value of ${basedir}
+     *  for all executions that come after the shade execution. This is often not what you want. This is considered
+     *  an open issue with this plugin.
+     */
+    private File dependencyReducedPomLocation;
 
     /**
      * When true, dependencies are kept in the pom but with scope 'provided'; when false,
@@ -285,36 +354,99 @@ public class ShadeMojo
      */
     private boolean createSourcesJar;
 
+    /**
+     * When true, dependencies will be stripped down on the class level to only the transitive hull required for the
+     * artifact. <em>Note:</em> Usage of this feature requires Java 1.5 or higher.
+     *
+     * @parameter default-value="false"
+     * @since 1.4
+     */
+    private boolean minimizeJar;
 
-    /** @throws MojoExecutionException  */
+    /**
+     * The path to the output file for the shaded artifact. When this parameter is set, the created archive will neither
+     * replace the project's main artifact nor will it be attached. Hence, this parameter causes the parameters
+     * {@link #finalName}, {@link #shadedArtifactAttached}, {@link #shadedClassifierName} and
+     * {@link #createDependencyReducedPom} to be ignored when used.
+     *
+     * @parameter
+     * @since 1.3
+     */
+    private File outputFile;
+
+    /**
+     * You can pass here the roleHint about your own Shader implementation plexus component.
+     *
+     * @parameter
+     * @since 1.6
+     */
+    private String shaderHint;
+
+    /**
+     * @since 1.6
+     */
+    private PlexusContainer plexusContainer;
+
+    public void contextualize( Context context )
+        throws ContextException
+    {
+        plexusContainer = (PlexusContainer) context.get( PlexusConstants.PLEXUS_KEY );
+    }
+
+    /**
+     * @throws MojoExecutionException
+     */
     public void execute()
         throws MojoExecutionException
     {
-        Set artifacts = new LinkedHashSet();
-        Set artifactIds = new LinkedHashSet();
-        Set sourceArtifacts = new LinkedHashSet();
 
-        if ( project.getArtifact().getFile() == null )
+        if ( shaderHint != null )
         {
-            getLog().error( "The project main artifact does not exist. This could have the following" );
-            getLog().error( "reasons:" );
-            getLog().error( "- You have invoked the goal directly from the command line. This is not" );
-            getLog().error( "  supported. Please add the goal to the default lifecycle via an" );
-            getLog().error( "  <execution> element in your POM and use \"mvn package\" to have it run." );
-            getLog().error( "- You have bound the goal to a lifecycle phase before \"package\". Please" );
-            getLog().error( "  remove this binding from your POM such that the goal will be run in" );
-            getLog().error( "  the proper phase." );
-            throw new MojoExecutionException( "Failed to create shaded artifact.",
-                                              new IllegalStateException( "Project main artifact does not exist." ) );
-        }
-        artifacts.add( project.getArtifact().getFile() );
-
-        if ( createSourcesJar )
-        {
-            File file = shadedSourcesArtifactFile();
-            if ( file.exists() )
+            try
             {
-                sourceArtifacts.add( file );
+                shader = (Shader) plexusContainer.lookup( Shader.ROLE, shaderHint );
+            }
+            catch ( ComponentLookupException e )
+            {
+                throw new MojoExecutionException(
+                    "unable to lookup own Shader implementation with hint:'" + shaderHint + "'", e );
+            }
+        }
+
+        Set<File> artifacts = new LinkedHashSet<File>();
+        Set<String> artifactIds = new LinkedHashSet<String>();
+        Set<File> sourceArtifacts = new LinkedHashSet<File>();
+
+        ArtifactSelector artifactSelector =
+            new ArtifactSelector( project.getArtifact(), artifactSet, shadedGroupFilter );
+
+        if ( artifactSelector.isSelected( project.getArtifact() ) && 
+                        !"pom".equals( project.getArtifact().getType() ) )
+        {
+            if ( invalidMainArtifact() )
+            {
+                getLog().error( "The project main artifact does not exist. This could have the following" );
+                getLog().error( "reasons:" );
+                getLog().error( "- You have invoked the goal directly from the command line. This is not" );
+                getLog().error( "  supported. Please add the goal to the default lifecycle via an" );
+                getLog().error( "  <execution> element in your POM and use \"mvn package\" to have it run." );
+                getLog().error( "- You have bound the goal to a lifecycle phase before \"package\". Please" );
+                getLog().error( "  remove this binding from your POM such that the goal will be run in" );
+                getLog().error( "  the proper phase." );
+                getLog().error( "- You removed the configuration of the maven-jar-plugin that produces the main artifact." );
+                throw new MojoExecutionException(
+                    "Failed to create shaded artifact, " + "project main artifact does not exist." );
+            }
+
+            artifacts.add( project.getArtifact().getFile() );
+
+            if ( createSourcesJar )
+            {
+                File file = shadedSourcesArtifactFile();
+                if ( file.isFile() )
+                {
+                    sourceArtifacts.add( file );
+                }
             }
         }
 
@@ -322,17 +454,22 @@ public class ShadeMojo
         {
             Artifact artifact = (Artifact) it.next();
 
-            if ( excludeArtifact( artifact ) )
+            if ( !artifactSelector.isSelected( artifact ) )
             {
                 getLog().info( "Excluding " + artifact.getId() + " from the shaded jar." );
 
                 continue;
             }
 
+            if ( "pom".equals( artifact.getType() ) )
+            {
+                getLog().info( "Skipping pom dependency " + artifact.getId() + " in the shaded jar." );
+                continue;
+            }
+
             getLog().info( "Including " + artifact.getId() + " in the shaded jar." );
 
             artifacts.add( artifact.getFile() );
-
             artifactIds.add( getId( artifact ) );
 
             if ( createSourcesJar )
@@ -345,18 +482,17 @@ public class ShadeMojo
             }
         }
 
-
-        File outputJar = shadedArtifactFileWithClassifier();
+        File outputJar = ( outputFile != null ) ? outputFile : shadedArtifactFileWithClassifier();
         File sourcesJar = shadedSourceArtifactFileWithClassifier();
 
         // Now add our extra resources
         try
         {
-            List filters = getFilters();
+            List<Filter> filters = getFilters();
 
-            List relocators = getRelocators();
+            List<Relocator> relocators = getRelocators();
 
-            List resourceTransformers = getResourceTransformers();
+            List<ResourceTransformer> resourceTransformers = getResourceTransformers();
 
             shader.shade( artifacts, outputJar, filters, relocators, resourceTransformers );
 
@@ -365,45 +501,69 @@ public class ShadeMojo
                 shader.shade( sourceArtifacts, sourcesJar, filters, relocators, resourceTransformers );
             }
 
-            if ( shadedArtifactAttached )
+            if ( outputFile == null )
             {
-                getLog().info( "Attaching shaded artifact." );
-                projectHelper.attachArtifact( project, project.getArtifact().getType(), shadedClassifierName, outputJar );
-                if ( createSourcesJar )
+                boolean renamed = false;
+
+                // rename the output file if a specific finalName is set
+                // but don't rename if the finalName is the <build><finalName>
+                // because this will be handled implicitly later
+                if ( finalName != null && finalName.length() > 0 && !finalName.equals(
+                    project.getBuild().getFinalName() ) )
                 {
-                    projectHelper.attachArtifact( project, "jar", shadedClassifierName + "-sources", sourcesJar );
-                }
-            }
+                    String finalFileName = finalName + "." + project.getArtifact().getArtifactHandler().getExtension();
+                    File finalFile = new File( outputDirectory, finalFileName );
+                    replaceFile( finalFile, outputJar );
+                    outputJar = finalFile;
 
-            else
-            {
-                getLog().info( "Replacing original artifact with shaded artifact." );
-                File file = shadedArtifactFile();
-                replaceFile( file, outputJar );
-
-                if ( createSourcesJar )
-                {
-                    file = shadedSourcesArtifactFile();
-
-                    replaceFile( file, sourcesJar );
-
-                    projectHelper.attachArtifact( project, "jar",
-                                                  "sources", file );
+                    renamed = true;
                 }
 
-                if ( createDependencyReducedPom )
+                if ( shadedArtifactAttached )
                 {
-                    createDependencyReducedPom( artifactIds );
+                    getLog().info( "Attaching shaded artifact." );
+                    projectHelper.attachArtifact( project, project.getArtifact().getType(), shadedClassifierName,
+                                                  outputJar );
+                    if ( createSourcesJar )
+                    {
+                        projectHelper.attachArtifact( project, "jar", shadedClassifierName + "-sources", sourcesJar );
+                    }
+                }
+                else if ( !renamed )
+                {
+                    getLog().info( "Replacing original artifact with shaded artifact." );
+                    File originalArtifact = project.getArtifact().getFile();
+                    replaceFile( originalArtifact, outputJar );
+
+                    if ( createSourcesJar )
+                    {
+                        File shadedSources = shadedSourcesArtifactFile();
+
+                        replaceFile( shadedSources, sourcesJar );
+
+                        projectHelper.attachArtifact( project, "jar", "sources", shadedSources );
+                    }
+
+                    if ( createDependencyReducedPom )
+                    {
+                        createDependencyReducedPom( artifactIds );
+                    }
                 }
             }
         }
         catch ( Exception e )
         {
-            throw new MojoExecutionException( "Error creating shaded jar.", e );
+            throw new MojoExecutionException( "Error creating shaded jar: " + e.getMessage(), e );
         }
     }
 
-    private void replaceFile( File oldFile, File newFile ) throws MojoExecutionException
+    private boolean invalidMainArtifact( )
+    {
+        return project.getArtifact().getFile() == null || ! project.getArtifact().getFile().isFile();                                       
+    }
+
+    private void replaceFile( File oldFile, File newFile )
+        throws MojoExecutionException
     {
         getLog().info( "Replacing " + oldFile + " with " + newFile );
 
@@ -473,11 +633,8 @@ public class ShadeMojo
     {
 
         Artifact resolvedArtifact =
-            artifactFactory.createArtifactWithClassifier( artifact.getGroupId(),
-                                                          artifact.getArtifactId(),
-                                                          artifact.getVersion(),
-                                                          "java-source",
-                                                          "sources" );
+            artifactFactory.createArtifactWithClassifier( artifact.getGroupId(), artifact.getArtifactId(),
+                                                          artifact.getVersion(), "java-source", "sources" );
 
         try
         {
@@ -499,54 +656,9 @@ public class ShadeMojo
         return null;
     }
 
-    private boolean excludeArtifact( Artifact artifact )
+    private List<Relocator> getRelocators()
     {
-        String id = getId( artifact );
-
-        // This is the case where we have only stated artifacts to include and no exclusions
-        // have been listed. We just want what we have asked to include.
-        if ( artifactSet != null && ( artifactSet.getExcludes() == null && artifactSet.getIncludes() != null )
-            && !includedArtifacts().contains( id ) )
-        {
-            return true;
-        }
-
-        if ( excludedArtifacts().contains( id ) )
-        {
-            return true;
-        }
-
-        if ( shadedGroupFilter != null && !artifact.getGroupId().startsWith( shadedGroupFilter ) )
-        {
-            return true;
-        }
-
-        return false;
-    }
-
-    private Set excludedArtifacts()
-    {
-        if ( artifactSet != null && artifactSet.getExcludes() != null )
-        {
-            return artifactSet.getExcludes();
-        }
-
-        return Collections.EMPTY_SET;
-    }
-
-    private Set includedArtifacts()
-    {
-        if ( artifactSet != null && artifactSet.getIncludes() != null )
-        {
-            return artifactSet.getIncludes();
-        }
-
-        return Collections.EMPTY_SET;
-    }
-
-    private List getRelocators()
-    {
-        List relocators = new ArrayList();
+        List<Relocator> relocators = new ArrayList<Relocator>();
 
         if ( relocations == null )
         {
@@ -557,57 +669,96 @@ public class ShadeMojo
         {
             PackageRelocation r = relocations[i];
 
-            relocators.add( new SimpleRelocator( r.getPattern(), r.getShadedPattern(), r.getExcludes() ) );
+            relocators.add( new SimpleRelocator( r.getPattern(), r.getShadedPattern(), r.getIncludes(), r.getExcludes(),
+                                                 r.isRawString() ) );
         }
 
         return relocators;
     }
 
-    private List getResourceTransformers()
+    private List<ResourceTransformer> getResourceTransformers()
     {
         if ( transformers == null )
         {
-            return Collections.EMPTY_LIST;
+            return Collections.emptyList();
         }
 
         return Arrays.asList( transformers );
     }
 
-    private List getFilters()
+    private List<Filter> getFilters()
+        throws MojoExecutionException
     {
-        List filters = new ArrayList();
+        List<Filter> filters = new ArrayList<Filter>();
+        List<SimpleFilter> simpleFilters = new ArrayList<SimpleFilter>();
 
-        if ( this.filters == null )
+        if ( this.filters != null && this.filters.length > 0 )
         {
-            return filters;
-        }
+            Map<Artifact, ArtifactId> artifacts = new HashMap<Artifact, ArtifactId>();
 
-        Map artifacts = new HashMap();
+            artifacts.put( project.getArtifact(), new ArtifactId( project.getArtifact() ) );
 
-        artifacts.put( getId( project.getArtifact() ), project.getArtifact().getFile() );
-
-        for ( Iterator it = project.getArtifacts().iterator(); it.hasNext(); )
-        {
-            Artifact artifact = (Artifact) it.next();
-
-            artifacts.put( getId( artifact ), artifact.getFile() );
-        }
-
-        for ( int i = 0; i < this.filters.length; i++ )
-        {
-            ArchiveFilter f = this.filters[i];
-
-            File jar = (File) artifacts.get( f.getArtifact() );
-
-            if ( jar == null )
+            for ( Iterator it = project.getArtifacts().iterator(); it.hasNext(); )
             {
-                getLog().info( "No artifact matching filter " + f.getArtifact() );
+                Artifact artifact = (Artifact) it.next();
 
-                continue;
+                artifacts.put( artifact, new ArtifactId( artifact ) );
             }
 
-            filters.add( new SimpleFilter( jar, f.getIncludes(), f.getExcludes() ) );
+            for ( int i = 0; i < this.filters.length; i++ )
+            {
+                ArchiveFilter filter = this.filters[i];
 
+                ArtifactId pattern = new ArtifactId( filter.getArtifact() );
+
+                Set<File> jars = new HashSet<File>();
+
+                for ( Iterator it = artifacts.entrySet().iterator(); it.hasNext(); )
+                {
+                    Map.Entry entry = (Map.Entry) it.next();
+
+                    if ( ( (ArtifactId) entry.getValue() ).matches( pattern ) )
+                    {
+                        Artifact artifact = (Artifact) entry.getKey();
+
+                        jars.add( artifact.getFile() );
+
+                        if ( createSourcesJar )
+                        {
+                            File file = resolveArtifactSources( artifact );
+                            if ( file != null )
+                            {
+                                jars.add( file );
+                            }
+                        }
+                    }
+                }
+
+                if ( jars.isEmpty() )
+                {
+                    getLog().info( "No artifact matching filter " + filter.getArtifact() );
+
+                    continue;
+                }
+
+                simpleFilters.add( new SimpleFilter( jars, filter.getIncludes(), filter.getExcludes() ) );
+            }
+        }
+
+        filters.addAll( simpleFilters );
+
+        if ( minimizeJar )
+        {
+            getLog().info( "Minimizing jar " + project.getArtifact() );
+
+            try
+            {
+                filters.add( new MinijarFilter( project, getLog(), simpleFilters ) );
+            }
+            catch ( IOException e )
+            {
+                throw new MojoExecutionException( "Failed to analyze class dependencies", e );
+            }
         }
 
         return filters;
@@ -616,9 +767,8 @@ public class ShadeMojo
     private File shadedArtifactFileWithClassifier()
     {
         Artifact artifact = project.getArtifact();
-        final String shadedName =
-            shadedArtifactId + "-" + artifact.getVersion() + "-" + shadedClassifierName + "."
-                + artifact.getArtifactHandler().getExtension();
+        final String shadedName = shadedArtifactId + "-" + artifact.getVersion() + "-" + shadedClassifierName + "."
+            + artifact.getArtifactHandler().getExtension();
         return new File( outputDirectory, shadedName );
     }
 
@@ -631,34 +781,15 @@ public class ShadeMojo
         return new File( outputDirectory, shadedName );
     }
 
-    private File shadedArtifactFile()
-    {
-        Artifact artifact = project.getArtifact();
-
-        String shadedName;
-
-        if ( finalName != null )
-        {
-            shadedName = finalName + "." + artifact.getArtifactHandler().getExtension();
-        }
-        else
-        {
-            shadedName = shadedArtifactId + "-" + artifact.getVersion() + "."
-                + artifact.getArtifactHandler().getExtension();
-        }
-
-        return new File( outputDirectory, shadedName );
-    }
-
     private File shadedSourcesArtifactFile()
     {
         Artifact artifact = project.getArtifact();
 
         String shadedName;
 
-        if ( finalName != null )
+        if ( project.getBuild().getFinalName() != null )
         {
-            shadedName = finalName + "-sources." + artifact.getArtifactHandler().getExtension();
+            shadedName = project.getBuild().getFinalName() + "-sources." + artifact.getArtifactHandler().getExtension();
         }
         else
         {
@@ -671,20 +802,26 @@ public class ShadeMojo
 
     // We need to find the direct dependencies that have been included in the uber JAR so that we can modify the
     // POM accordingly.
-    private void createDependencyReducedPom( Set artifactsToRemove )
+    private void createDependencyReducedPom( Set<String> artifactsToRemove )
         throws IOException, DependencyTreeBuilderException, ProjectBuildingException
     {
         Model model = project.getOriginalModel();
-        List dependencies = new ArrayList();
+        List<Dependency> dependencies = new ArrayList<Dependency>();
 
         boolean modified = false;
 
-        List transitiveDeps = new ArrayList();
+        List<Dependency> transitiveDeps = new ArrayList<Dependency>();
 
         for ( Iterator it = project.getArtifacts().iterator(); it.hasNext(); )
         {
             Artifact artifact = (Artifact) it.next();
 
+            if ( "pom".equals( artifact.getType() ) )
+            {
+                // don't include pom type dependencies in dependency reduced pom
+                continue;
+            }
+
             //promote
             Dependency dep = new Dependency();
             dep.setArtifactId( artifact.getArtifactId() );
@@ -702,20 +839,20 @@ public class ShadeMojo
 
             transitiveDeps.add( dep );
         }
-        List origDeps = project.getDependencies();
+        List<Dependency> origDeps = project.getDependencies();
 
         if ( promoteTransitiveDependencies )
         {
             origDeps = transitiveDeps;
         }
 
-        for ( Iterator i = origDeps.iterator(); i.hasNext(); )
+        for ( Iterator<Dependency> i = origDeps.iterator(); i.hasNext(); )
         {
-            Dependency d = (Dependency) i.next();
+            Dependency d = i.next();
 
             dependencies.add( d );
 
-            String id = d.getGroupId() + ":" + d.getArtifactId();
+            String id = getId( d );
 
             if ( artifactsToRemove.contains( id ) )
             {
@@ -740,7 +877,13 @@ public class ShadeMojo
 
                 model.setDependencies( dependencies );
 
-                File f = new File( outputDirectory, "dependency-reduced-pom.xml" );
+                if ( dependencyReducedPomLocation == null )
+                {
+                    // MSHADE-123: We can't default to 'target' because it messes up uses of ${project.basedir}
+                    dependencyReducedPomLocation = new File ( project.getBasedir(), "dependency-reduced-pom.xml" );
+                }
+                
+                File f = dependencyReducedPomLocation;
                 if ( f.exists() )
                 {
                     f.delete();
@@ -748,57 +891,78 @@ public class ShadeMojo
 
                 Writer w = WriterFactory.newXmlWriter( f );
 
-                PomWriter.write( w, model, true );
+                String origRelativePath = null;
+                String replaceRelativePath = null;
+                if ( model.getParent() != null)
+                {
+                    origRelativePath = model.getParent().getRelativePath();
+                    
+                }
+                replaceRelativePath = origRelativePath;
 
-                w.close();
+                if ( origRelativePath == null )
+                {
+                    origRelativePath = "../pom.xml";
+                }
+
+                if ( model.getParent() != null ) 
+                {
+                    File parentFile = new File( project.getBasedir(), model.getParent().getRelativePath() ).getCanonicalFile();
+                    if ( !parentFile.isFile() )
+                    {
+                        parentFile = new File( parentFile, "pom.xml");
+                    }
+                    
+                    parentFile = parentFile.getCanonicalFile();
+                    
+                    String relPath = RelativizePath.convertToRelativePath( parentFile, f );
+                    model.getParent().setRelativePath( relPath );
+                }
+                
+                try
+                {
+                    PomWriter.write( w, model, true );
+                }
+                finally
+                {
+                    if ( model.getParent() != null )
+                    {
+                        model.getParent().setRelativePath( replaceRelativePath );
+                    }
+                    w.close();
+                }
 
                 MavenProject p2 = mavenProjectBuilder.build( f, localRepository, null );
                 modified = updateExcludesInDeps( p2, dependencies, transitiveDeps );
 
             }
 
-            //copy the dependecy-reduced-pom.xml to the basedir where
-            //we'll set the file for the project to it.  We cannot set
-            //it to the real version in "target" as then ${basedir} gets
-            //messed up.   We'll delete this file on exit to make
-            //sure it gets cleaned up.
-            File f = new File( project.getBasedir(), "dependency-reduced-pom.xml" );
-            File f2 = new File( outputDirectory, "dependency-reduced-pom.xml" );
-            if ( f.exists() )
-            {
-                f.delete();
-            }
-            FileUtils.copyFile( f2, f );
-            FileUtils.forceDeleteOnExit( f );
-            project.setFile( f );
+            project.setFile( dependencyReducedPomLocation );
         }
     }
-
+    
     private String getId( Artifact artifact )
     {
-        if ( artifact.getClassifier() == null || "jar".equals( artifact.getClassifier() ) )
-        {
-            return artifact.getGroupId() + ":" + artifact.getArtifactId();
-        }
-        else
-        {
-            return artifact.getGroupId() + ":" + artifact.getArtifactId() + ":" + artifact.getClassifier();
-        }
+        return getId( artifact.getGroupId(), artifact.getArtifactId(), artifact.getType(), artifact.getClassifier() );
     }
 
+    private String getId( Dependency dependency )
+    {
+        return getId( dependency.getGroupId(), dependency.getArtifactId(), dependency.getType(),
+                      dependency.getClassifier() );
+    }
 
-    public boolean updateExcludesInDeps( MavenProject project,
-                                         List dependencies,
-                                         List transitiveDeps )
+    private String getId( String groupId, String artifactId, String type, String classifier )
+    {
+        return groupId + ":" + artifactId + ":" + type + ":" + ( ( classifier != null ) ? classifier : "" );
+    }
+
+    public boolean updateExcludesInDeps( MavenProject project, List<Dependency> dependencies, List<Dependency> transitiveDeps )
         throws DependencyTreeBuilderException
     {
-        DependencyNode node = dependencyTreeBuilder.buildDependencyTree(
-                                                  project,
-                                                  localRepository,
-                                                  artifactFactory,
-                                                  artifactMetadataSource,
-                                                  null,
-                                                  artifactCollector );
+        DependencyNode node = dependencyTreeBuilder.buildDependencyTree( project, localRepository, artifactFactory,
+                                                                         artifactMetadataSource, null,
+                                                                         artifactCollector );
         boolean modified = false;
         Iterator it = node.getChildren().listIterator();
         while ( it.hasNext() )
@@ -822,9 +986,9 @@ public class ShadeMojo
                     boolean found = false;
                     for ( int x = 0; x < transitiveDeps.size(); x++ )
                     {
-                        Dependency dep = (Dependency) transitiveDeps.get( x );
-                        if ( dep.getArtifactId().equals( n3.getArtifact().getArtifactId() )
-                            && dep.getGroupId().equals( n3.getArtifact().getGroupId() ) )
+                        Dependency dep = transitiveDeps.get( x );
+                        if ( dep.getArtifactId().equals( n3.getArtifact().getArtifactId() ) && dep.getGroupId().equals(
+                            n3.getArtifact().getGroupId() ) )
                         {
                             found = true;
                         }
@@ -835,7 +999,7 @@ public class ShadeMojo
                     {
                         for ( int x = 0; x < dependencies.size(); x++ )
                         {
-                            Dependency dep = (Dependency) dependencies.get( x );
+                            Dependency dep = dependencies.get( x );
                             if ( dep.getArtifactId().equals( n2.getArtifact().getArtifactId() )
                                 && dep.getGroupId().equals( n2.getArtifact().getGroupId() ) )
                             {
