@@ -19,8 +19,11 @@ package org.apache.maven.plugins.shade.resource;
  * under the License.
  */
 
+import org.apache.maven.plugins.shade.relocation.Relocator;
+
 import java.io.IOException;
 import java.io.InputStream;
+import java.util.List;
 import java.util.jar.JarOutputStream;
 
 /** @author Jason van Zyl */
@@ -28,7 +31,7 @@ public interface ResourceTransformer
 {
     boolean canTransformResource( String resource );
 
-    void processResource( InputStream is )
+    void processResource( String resource, InputStream is, List<Relocator> relocators )
         throws IOException;
 
     boolean hasTransformedResource();
